package com.muyu.cloud.etl.service.impl;

import com.muyu.Hikari.HikariPool;
import com.muyu.cloud.etl.service.ProductService;
import com.muyu.cloud.etl.service.SourceService;
import com.muyu.cloud.etl.service.TableInfoService;
import com.muyu.domain.DataValue;
import com.muyu.domain.Source;
import com.muyu.domain.TableInfo;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Date;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

@Slf4j
@Service
public class ProductServiceImpl implements ProductService {
    @Autowired
    private SourceService sourceService;
    @Autowired
    private TableInfoService tableInfoService;

    public int addProduct(Long basicId, Long tableId, DataValue[][] listList) {
//        TableInfo tableInfoDataSources = tableInfoService.getById(basicId);
//        Long basicId1 = tableInfoDataSources.getBasicId();
        Source dataSources = sourceService.getById(basicId);
        TableInfo tableInfo = tableInfoService.getById(tableId);
        String tableName = tableInfo.getTableName();

//        HikariDataSource hikariDataSource = getHikariDataSource(dataSources);

        HikariDataSource hikariDataSource = HikariPool.getHikariDataSource(dataSources);

        ExecutorService executorService = Executors.newFixedThreadPool(4);

        AtomicInteger addCount = new AtomicInteger();

        Connection connection = null;

        try {
            connection = hikariDataSource.getConnection();
            // 遍历外部列表
            for (DataValue[] dataValues : listList) {
                log.info("bbbbbb");
                for (DataValue dataValue : dataValues) {
                    log.info("man{}",dataValue);
                }
            }
            log.info("aaaaaaaaaaaaaaaaaaaaaa");
            for (DataValue[] dataValueList : listList) {
                log.info("便利化后的外部列表的值是{}",dataValueList.toString());
                Connection finalConnection = connection;
                executorService.submit(() -> {
                    try {
                        addCount.addAndGet(insertRow(finalConnection, tableName, dataValueList));
                    } catch (SQLException e) {
                        // 记录异常
                        e.printStackTrace();
                    }
                });
            }

            executorService.shutdown();
            executorService.awaitTermination(1, TimeUnit.HOURS);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException("Thread interrupted", e);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            closeConnection(connection);
        }

        return addCount.get();
    }



//    private int insertRow(Connection conn, String tableName, DataValue[] dataValueList) throws SQLException {
//        // 获取当前行的所有字段名
//        StringBuilder columns = new StringBuilder("(");
//        StringBuilder values = new StringBuilder("VALUES (");
//
//        // 构建字段名和占位符
//        for (DataValue dataValue : dataValueList) {
//            String key = dataValue.getKey();
//            columns.append(key).append(", ");
//            values.append("?, ");
//        }
//        // 删除最后一个逗号和空格
//        columns.delete(columns.length() - 2, columns.length());
//        values.delete(values.length() - 2, values.length());
//
//        // 完成 SQL 插入语句
//        String sql = "INSERT INTO " + tableName + " (" + columns.toString() + ") " + values.toString();
//
//        int addCount = 0;
//        try (PreparedStatement ps = conn.prepareStatement(sql)) {
//            // 设置参数
//            int index = 1;
//            for (DataValue dataValue : dataValueList) {
//                Object value = dataValue.getValue();
//                if (value instanceof String) {
//                    ps.setString(index, (String) value);
//                } else if (value instanceof Integer) {
//                    ps.setInt(index, (Integer) value);
//                } else if (value instanceof Double) {
//                    ps.setDouble(index, (Double) value);
//                } else if (value instanceof Date) {
//                    ps.setDate(index, new java.sql.Date(((Date) value).getTime()));
//                } else {
//                    // 其他类型的处理
//                    ps.setObject(index, value);
//                }
//                index++;
//            }
//            // 执行插入操作
//            addCount = ps.executeUpdate();
//        }
//        return addCount;
//    }


    public int insertRow(Connection conn, String tableName, DataValue[] dataValueList) throws SQLException {

        // 获取当前行的所有字段名
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("VALUES (");

        // 构建字段名和占位符
     for (DataValue dataValue : dataValueList) {
            String key = dataValue.getKey();
            columns.append(key).append(", ");
            values.append("?, ");
        }
        // 删除最后一个逗号和空格
    columns.delete(columns.length() - 2, columns.length());
    values.delete(values.length() - 2, values.length());

        // 完成 SQL 插入语句
        String sql = "INSERT INTO " + tableName + " " + columns.toString() + ") " + values.toString() + ")";
        log.info("添加语句{}",sql);

        int addCount = 0;

        try {
            conn.setAutoCommit(false);

            try (

                    PreparedStatement ps = conn.prepareStatement(sql)) {


                // 循环设置参数并执行插入
                for (DataValue dataValue : dataValueList) {
                    log.info("循环的值为："+dataValue.toString());
                    int index = 1;
                    Object obj = dataValue.getValue();
                    if (obj instanceof String) {
                        ps.setString(index++, (String) obj);
                        log.info("类型为String,值{}",obj);
                    } else if (obj instanceof Integer) {
                        ps.setInt(index++, (Integer) obj);
                        log.info("类型为Integer,值{}",obj);
                    } else if (obj instanceof Double) {
                        ps.setDouble(index++, (Double) obj);
                        log.info("类型为Double,值{}",obj);
                    } else if (obj instanceof Date) {
                        ps.setDate(index++, new java.sql.Date(((Date) obj).getTime()));
                        log.info("类型为Date,值{}",obj);
                    } else if (obj instanceof Boolean) {
                        ps.setBoolean(index++, (Boolean) obj);
                        log.info("类型为Boolean,值{}",obj);
                    } else if (obj instanceof Float) {
                        ps.setFloat(index++, (Float) obj);
                        log.info("类型为Float,值{}",obj);
                    } else if (obj instanceof Long) {
                        ps.setLong(index++, (Long) obj);
                        log.info("类型为Long,值{}",obj);
                    } else {
                        ps.setObject(index++, obj);
                        log.info("类型为obj,值{}",obj);
                    }
                }
                ps.addBatch();
                // 执行批量插入操作
                int[] ints = ps.executeBatch();

                for (int anInt : ints) {
                    log.info("插入成功的数据有"+anInt);
                }

                conn.commit();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }


    return addCount;
    }


    private void closeConnection(Connection conn) {
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }


//    public static HikariDataSource instance = null;
//    @NotNull
//    public static synchronized HikariDataSource getHikariDataSource(Source dataSources) {
//        if(instance == null) {
//            HikariConfig hikariConfig = new HikariConfig();
//            hikariConfig.setPoolName("HikariCP 连接池");
//            hikariConfig.setDataSourceClassName(dataSources.getDriverName());
//            hikariConfig.addDataSourceProperty("user", dataSources.getUsername());
//            hikariConfig.addDataSourceProperty("password", dataSources.getPassword());
//            hikariConfig.addDataSourceProperty("url", dataSources.getUrl());
//            hikariConfig.setMaximumPoolSize(15);
//
//            instance = new HikariDataSource(hikariConfig);
//        }
//        return instance;
//    }



}
